/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License specifically applicable to
 * this software at http://developer.yahoo.com/yslow/license.txt, or the general
 * form at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is: YSlow
 *
 * The Initial Developer of the Original Code is Yahoo! Inc.
 *
 * Copyright (C) 2007, Yahoo! Inc. All Rights Reserved.
 *
 * Contributor(s):
 * ***** END LICENSE BLOCK ***** */

//Tools Class object in the YSLOW namespace.
if ( typeof YSLOW.Tools == "undefined") {
	YSLOW.Tools = {};
}

// Combine all the JS into one window.
YSLOW.Tools.loadAllJS = function(beautified) {
	var sHtml = "";
	var sJSCode = "";

    var toc = '';

	// Iterate over the external JS files.
	var aComponents = FirebugContext.yslowContext.getComponents("js");
	for ( var i = 0; i < aComponents.length; i++ ) {
		var compObj = aComponents[i];
		if ( compObj['responseText'] ) {
			sJSCode += '<div id="ext-' + i + '" style="color: blue">\n\n////////////////////////////////////////////////////////////////////////////////\n// ' +
                       YSLOW.Utils.escapeHtml(compObj['url']) + "\n////////////////////////////////////////////////////////////////////////////////\n</div>";

            sJSCode += YSLOW.Utils.escapeHtml((beautified) ? js_beautify(compObj['responseText']) : compObj['responseText']);

            toc += '<li><a href="about:blank#ext-' + i + '">' + YSLOW.Utils.escapeHtml(compObj['url']) + '</a></li>';
		}
	}

	// Iterate over the inline SCRIPT blocks.
	var aScripts = FirebugContext.yslowContext.browserDoc.getElementsByTagName("script");
	var iScript = 0;
	for ( var i = 0; i < aScripts.length; i++ ) {
		var script = aScripts[i];
		if ( !script.src ) {  // avoid external script objects
            iScript++;
            sJSCode += '<div id="in-' + i + '" style="color: red">\n\n////////////////////////////////////////////////////////////////////////////////\n// ' +
                       "inline script block #" + parseInt(iScript, 10) + "\n////////////////////////////////////////////////////////////////////////////////\n</div>";
            sJSCode += YSLOW.Utils.escapeHtml((beautified) ? js_beautify(script.textContent) : script.textContent);
            toc += '<li><a style="color: red" href="about:blank#in-' + i + '">inline script block #' + parseInt(iScript, 10) + '</a></li>';
		}
	}

    sHtml += "/*<h1>";
    sHtml += ((beautified) ? 'Beautified' : 'All') + " JavaScript for: ";
    sHtml += YSLOW.Utils.escapeHtml(FirebugContext.yslowContext.browserDoc.location.href) + "</h1>Table of Contents:";
    sHtml += '<ol>' + toc + '</ol>*/';
	sHtml += "<pre>";
    sHtml += sJSCode;
    sHtml += "</pre>";

    var doctitle = ((beautified) ? 'Beautified' : 'All') + 'JavaScript for: ' + YSLOW.Utils.escapeHtml(FirebugContext.yslowContext.browserDoc.location.href.substring(0, 25));

    var doc = YSLOW.Tools.getNewDoc();
    doc.title = doctitle;
    doc.body.innerHTML = sHtml;

}


// Combine all the CSS into one window.
YSLOW.Tools.loadAllCSS = function() {

	var sHtml = '', sCode = '', toc = '';

	// Iterate over the external files.
	var aComponents = FirebugContext.yslowContext.getComponents("css");
	for ( var i = 0; i < aComponents.length; i++ ) {
		var compObj = aComponents[i];
		if ( compObj['responseText'] ) {
            sCode +='<div id="ext-' + i + '" style="color: blue">'+
                       '\n\n/*******************************************************************************\n  ' +
                       YSLOW.Utils.escapeHtml(compObj['url']) +
					   "\n*******************************************************************************/\n</div>" +
                       YSLOW.Utils.escapeHtml(compObj['responseText']);

            toc += '<li><a href="about:blank#ext-' + i + '">' + YSLOW.Utils.escapeHtml(compObj['url']) + '</a></li>';
		}
	}

	// Iterate over the inline STYLE blocks.
	var aElements = FirebugContext.yslowContext.browserDoc.getElementsByTagName("style");
	var bFirstInline = true;
	for ( var i = 0; i < aElements.length; i++ ) {
		var elem = aElements[i];
		if ( elem.innerHTML ) {  // avoid external elem objects
			if ( bFirstInline ) {
                sCode += '<div id="in-' + i + '" style="color: red">' +
                           "\n\n/************************************************\n " +
					       " inline style blocks" + "\n************************************************/\n</div>";
				bFirstInline = false;
                toc += '<li><a style="color: red" href="about:blank#in-' + i + '">inline style blocks</a></li>';
			}
            sCode += YSLOW.Utils.escapeHtml(elem.textContent);
		}
	}


    sHtml += "/*<h1>All CSS for: " + YSLOW.Utils.escapeHtml(FirebugContext.yslowContext.browserDoc.location.href) + "</h1>";

    sHtml += 'Table of Contents: <ol>' + toc + '</ol>*/';
	sHtml += "<pre>" + sCode + "</pre>";

    var doctitle = 'All CSS for: ' + YSLOW.Utils.escapeHtml(FirebugContext.yslowContext.browserDoc.location.href.substring(0, 25));

    var doc = YSLOW.Tools.getNewDoc();
    doc.title = doctitle;
    doc.body.innerHTML = sHtml;


}


YSLOW.Tools.printable = function() {

    FirebugContext.yslowContext.bPrintable = true;

    // read the yslow logo as a binary stream, then base64 and use as data URI
    netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
    var req = new XMLHttpRequest();
    req.open('GET', 'chrome://yslow/content/logo_32x32.png', false);
    req.overrideMimeType('text/plain; charset=x-user-defined');
    req.send(null);
    var logo = req.responseText;
    var logobytes = '';
    for (var i = 0; i < logo.length; i++) {
        logobytes += String.fromCharCode(logo.charCodeAt(i) & 0xff);
    }
    var data_uri = 'data:image/png;base64,' + btoa(logobytes);


    var cnt = '<div style="font-size: 1.5em; color: #E05000; margin: 2px;"><img style="padding-right: 5px" src="'+ data_uri +'" align="absbottom"/>YSlow for Firebug</div>'+
        '<hr />'+

        //Print the website address:
        '<div style="margin-bottom: 10px; font-size: 1.5em;">' +
        '<span>URL:</span>' +
        '<span style="margin-left: 10px">' + YSLOW.Utils.prettyAnchor(FirebugContext.window.document.location.href,FirebugContext.window.document.location.href) + '</span>' +
        '</div>'+

        '<div id=yslowDiv>'+	//yslowDiv start

        // performance
        '<div class=peelerText style="font-size: 1.5em;" style="margin-top:  10px">Performance</div>' +
        '<div id=borderDiv class=peelerDiv style="width: auto; height: auto; position: relative; margin-bottom: 20px; border: 4px #F0F0F0 ridge; padding-bottom: 10px;">\n' +
        FirebugContext.yslowContext.genPerformance() +
        '</div>\n' +

        // stats
        '<div class=peelerText style="font-size: 1.5em;">Stats</div>' +
        '<div id=borderDiv class=peelerDiv style="width: auto; height: auto; position: relative; margin-bottom: 20px; border: 4px #F0F0F0 ridge; padding-bottom: 10px;">\n' +
        FirebugContext.yslowContext.genStats() +
        '</div>\n' +

        // components
        '<div class=peelerText style="font-size: 1.5em;">Components</div>' +
        '<div id=borderDiv class=peelerDiv style="width: auto; height: auto; position: relative; margin-bottom: 20px; border: 4px #F0F0F0 ridge; padding-bottom: 10px;">\n' +
        FirebugContext.yslowContext.genComponents() +
        '</div>\n' +
        '</div>\n' + //yslowDiv END
        '<br /><hr />'+
        '<div style="text-align: center; font-size: 10pt; margin-top: 20px;">Copyright &copy; 2007 Yahoo! Inc. All rights reserved.</div>'+
        '';


    //If we were generating performance stats for a printable version, we should reset this flag after we are done.
    //This is required so that performance output on the FF panel remains unexpanded and correctly formated.
    FirebugContext.yslowContext.bPrintable = false;

    // remember the context, for some reason it's easily forgotten
    var cntx = FirebugContext.yslowContext;

    var doc = YSLOW.Tools.getNewDoc();
    doc.body.innerHTML = cnt;

    // add styling
    var URI = 'chrome://yslow/content/yslow.css';
    var req = new XMLHttpRequest();
    req.open('GET', URI, false);
    req.overrideMimeType('text/css');
    req.send(null);
    var css = req.responseText;
    css += '#borderDiv {background: #FFFFFF;}';
    var h = doc.getElementsByTagName('head')[0];
    var l  = doc.createElement("style");
    l.setAttribute("type", "text/css");
    l.appendChild(doc.createTextNode(css));
    h.appendChild(l);

    // plot charts
    cntx.plotComponents.call(cntx, doc);
}



YSLOW.Tools.getNewDoc = function(){
    var generatedPage = null;
    var request       = new XMLHttpRequest();
    getBrowser().selectedTab = getBrowser().addTab('about:blank');
    generatedPage = window;
    request.open("get", "about:blank", false);
    request.overrideMimeType('text/html');
    request.send(null);
    return generatedPage.content.document;
}

YSLOW.Tools.loadCSS = function(doc, url) {

    var req = new XMLHttpRequest();
    req.open('GET', url, false);
    req.send(null);
    var css = req.responseText;

    var l  = d.createElement("style");
    l.setAttribute("type", "text/css");
    l.appendChild(d.createTextNode(css));
    doc.getElementsByTagName('head')[0].appendChild(l);
}
